import json


def load_terraform_outputs_file(file_path):
    """load_terraform_outputs_file Import JSON file of terraform outputs

    Arguments:
        file_path {str} -- File to be loaded

    Returns:
        [dict] -- {variable_name: variable_value}
    """
    try:
        state = json.loads(open(file_path).read().strip())
        return {k: data["value"] for k, data in state.items()}
    except json.decoder.JSONDecodeError:
        return {}



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class LicenseContainerDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"containers_run_count": "int", "containers_image_count": "int"}

    attribute_map = {
        "containers_run_count": "containers_run_count",
        "containers_image_count": "containers_image_count",
    }

    def __init__(
        self, containers_run_count=None, containers_image_count=None
    ):  # noqa: E501
        """LicenseContainerDetails - a model defined in OpenAPI"""  # noqa: E501

        self._containers_run_count = None
        self._containers_image_count = None
        self.discriminator = None

        if containers_run_count is not None:
            self.containers_run_count = containers_run_count
        if containers_image_count is not None:
            self.containers_image_count = containers_image_count

    @property
    def containers_run_count(self):
        """Gets the containers_run_count of this LicenseContainerDetails.  # noqa: E501


        :return: The containers_run_count of this LicenseContainerDetails.  # noqa: E501
        :rtype: int
        """
        return self._containers_run_count

    @containers_run_count.setter
    def containers_run_count(self, containers_run_count):
        """Sets the containers_run_count of this LicenseContainerDetails.


        :param containers_run_count: The containers_run_count of this LicenseContainerDetails.  # noqa: E501
        :type: int
        """

        self._containers_run_count = containers_run_count

    @property
    def containers_image_count(self):
        """Gets the containers_image_count of this LicenseContainerDetails.  # noqa: E501


        :return: The containers_image_count of this LicenseContainerDetails.  # noqa: E501
        :rtype: int
        """
        return self._containers_image_count

    @containers_image_count.setter
    def containers_image_count(self, containers_image_count):
        """Sets the containers_image_count of this LicenseContainerDetails.


        :param containers_image_count: The containers_image_count of this LicenseContainerDetails.  # noqa: E501
        :type: int
        """

        self._containers_image_count = containers_image_count

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LicenseContainerDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import asyncio
import json
import math
import time

from copy import deepcopy
from typing import Dict, Tuple, List, Callable, Union, Awaitable

from cohesivenet.log_util import scrub_sensitive


def run_parallel(*coroutines):
    loop = asyncio.get_event_loop()
    results = loop.run_until_complete(asyncio.gather(*coroutines))
    return results


def run_pipe(init_data, steps: List[Tuple[str, Callable]]):
    data = deepcopy(init_data)
    total_steps = len(steps)
    print(
        "Running pipe [steps=%s] [inputs=%s]"
        % (total_steps, scrub_sensitive(init_data))
    )

    for step_i, (step_name, func) in enumerate(steps):
        step_num = step_i + 1
        print(
            "Running step [step=%s/%s] [name=%s]" % (step_num, total_steps, step_name)
        )
        response = func(data)
        outputs = response.get("outputs", {})
        data.update(outputs)
        print("Step %s/%s finished. Outputs: %s" % (step_num, total_steps, outputs))
    return data


def run_pipe_async(
    init_data, steps: List[Tuple[str, Union[Callable, List[Awaitable]]]]
):
    """Run pipeline of step functions, running in parallel if

    Arguments:
        init_data {[type]} -- [description]
        steps {List[Tuple[str, Union[Callable, List[Awaitable]]]]} -- [description]

    Returns:
        [type] -- [description]
    """
    data = deepcopy(init_data)
    total_steps = len(steps)
    print(
        "Running pipe [steps=%s] [inputs=%s]"
        % (total_steps, scrub_sensitive(init_data))
    )
    for step_i, (step_name, step_func) in enumerate(steps):
        step_num = step_i + 1
        print(
            "Running pipe step [step=%s/%s] [name=%s]"
            % (step_num, total_steps, step_name)
        )
        if type(step_func) is list:
            print("Running async substeps")
            step_responses = run_parallel(*(func(data) for func in step_func))
            print("Substeps finished. Outputs: %s" % step_responses)
            for response in step_responses:
                data.update(response.get("outputs", {}))
        else:
            response = step_func(data)
            outputs = response.get("outputs", {})
            data.update(outputs)
            print("Step %s/%s finished. Outputs: %s" % (step_num, total_steps, outputs))
    return data


def take_keys(keys: List[str], data_dict: Dict):
    """Take keys from dict

    Arguments:
        keys {List[str]} -- Keys it include in output dict
        data_dict {Dict}

    Returns:
        [Dict]
    """
    return {k: v for k, v in data_dict.items() if k in keys}


def flatten_dict(d, prefix=None, joinchar="__"):
    """flatten_dict Flatten nested dictionary, joining paths into single string key

    Arguments:
        d {[type]} -- [description]

    Keyword Arguments:
        prefix {str} -- Prefix string for joining keys (default: {None})
        joinchar {str} -- String for joining nested keys (default: {'__'})

    Returns:
        [Dict] -- Dict of depth 1
    """
    key_value_pairs = {}

    def _prefix(k):
        return k if not prefix else "%s%s%s" % (prefix, joinchar, k)

    for k, v in d.items():
        if type(v) is dict:
            key_value_pairs.update(
                flatten_dict(v, prefix=_prefix(k), joinchar=joinchar)
            )
        else:
            key_value_pairs[_prefix(k)] = v
    return key_value_pairs


def unflatten_dict(d, splitchar="__"):
    """unflatten_dict Build nested dictionary based on keys

    Arguments:
        d {Dict}

    Keyword Arguments:
        splitchar {str} -- str marking nested dictionary (default: {'__'})

    Returns:
        [Dict]
    """
    resp = {}
    for flatkey, v in d.items():
        keyparts = flatkey.split(splitchar)
        _target = resp
        final_key = keyparts[-1]
        for key in keyparts[:-1]:
            if key not in _target:
                _target[key] = {}
            _target = _target[key]
        _target[final_key] = v
    return resp


def map_type(s, expected_type, strict=True):
    _def_return_val = s
    try:
        if type(s) is list:
            return s
        elif "list" in str(expected_type).lower():
            _def_return_val = []
            return json.loads(s)
        elif not s or s.lower() in ("none", ""):
            return None
        elif expected_type is bool:
            return s.lower() == "true"

        return expected_type(s)
    except ValueError:
        if not strict:
            return _def_return_val
        raise


def get_path(data_dict, key_path, fail=False):
    """get_path

    Arguments:
        data_dict {dict} -- [description]
        key_path {str or List of strings} -- a.b.c or ['a', 'b', 'c']

    Keyword Arguments:
        fail {bool} -- Raise exception if does not exist (default: {False})

    Raises:
        Exception: Generic not found exception

    Returns:
        [any] -- value at key path
    """
    _target = data_dict
    steps = key_path if type(key_path) is list else key_path.split(".")
    for step in steps:
        if type(_target) is not dict or step not in _target:
            if fail:
                raise Exception('Path "%s" does not exist' % key_path)
            return None
        _target = _target.get(step)
    return _target


def map_dict_values(func_map, data_dict):
    return {k: func_map.get(k, lambda x: x)(v) for k, v in data_dict.items()}


def map_dict_keypaths(key_map, data_dict):
    updates = {
        new_key: get_path(data_dict, key_path) for key_path, new_key in key_map.items()
    }
    return {**data_dict, **updates}


def partition_list_groups(object_list, number_partitions):
    """Partition list of objects into groups

    Arguments:
        object_list {List[Any]}
        number_partitions {int} -- [description]

    Returns:
        [List[List[Any]]]
    """
    if number_partitions <= 1:
        return object_list

    object_count = len(object_list)
    partition_size = math.floor(object_count / number_partitions)
    leftovers = object_count % number_partitions
    leftover_set = object_list[-leftovers:] if leftovers else []
    rounded_list = object_list[:-leftovers] if leftovers else object_list

    return [
        rounded_list[i * partition_size: (i + 1) * partition_size]
        + ([leftover_set[i]] if len(leftover_set) > i else [])
        for i in range(number_partitions)
    ]


def partition_list_ratios(object_list, partition_ratios):
    """Partition list of objects into groups based on ratios list

    Arguments:
        object_list {List[Any]}
        partition_ratios {List[float]}

    Returns:
        Dict[str, List[Any]] - {
            '0.45': [...],
            '0.25': [...],
            '0.30': [...]
        }
    """
    assert math.isclose(sum(partition_ratios), 1.0), "Ratios must sum to 1"

    number_partitions = len(partition_ratios)
    if number_partitions <= 1:
        return object_list

    partition_sizes = [round(r * len(object_list)) for r in partition_ratios]
    _cursor = 0
    partitions = {}
    for i, size in enumerate(partition_sizes):
        partitions[str(partition_ratios[i])] = object_list[_cursor: (_cursor + size)]
        _cursor += size
    return partitions


def random_timestamp_filename(file_type=None):
    timestr = str(time.time()).replace(".", "_")
    if file_type:
        return "%s.%s" % (timestr, file_type)
    return timestr
